@@ -40,17 +40,25 @@
 #define PidTagMailboxOwnerName PR_USER_NAME_UNICODE
 #endif
 
+#define STR_ACCOUNTS_COMBO		"e-mapi-accounts-combo"
 #define STR_USER_NAME_SELECTOR_ENTRY	"e-mapi-name-selector-entry"
 #define STR_FOLDER_NAME_COMBO		"e-mapi-folder-name-combo"
+#define STR_SUBFOLDERS_CHECK		"e-mapi-subfolders-check"
 #define STR_MAPI_CAMEL_SESSION		"e-mapi-camel-session"
-#define STR_MAPI_CAMEL_STORE		"e-mapi-camel-store"
 #define STR_MAPI_DIRECT_USER_NAME	"e-mapi-direct-user-name"
 
+enum {
+	COLUMN_UID = 0,
+	COLUMN_DISPLAY_NAME,
+	COLUMN_STORE
+};
+
 static gboolean
 add_foreign_folder_to_camel (CamelMapiStore *mapi_store,
 			     const gchar *foreign_username,
 			     mapi_id_t folder_id,
 			     mapi_id_t parent_fid,
+			     gboolean include_subfolders,
 			     const gchar *display_username,
 			     const gchar *display_foldername,
 			     GError **perror)
@@ -122,7 +130,8 @@ add_foreign_folder_to_camel (CamelMapiStore *mapi_store,
 
 		if (camel_mapi_store_summary_add_from_full (mapi_store->summary, path, folder_id, parent_fid,
 			CAMEL_STORE_INFO_FOLDER_SUBSCRIBED | CAMEL_FOLDER_NOCHILDREN | CAMEL_FOLDER_SUBSCRIBED,
-			CAMEL_MAPI_STORE_FOLDER_FLAG_FOREIGN | CAMEL_MAPI_STORE_FOLDER_FLAG_MAIL,
+			CAMEL_MAPI_STORE_FOLDER_FLAG_FOREIGN | CAMEL_MAPI_STORE_FOLDER_FLAG_MAIL |
+			(include_subfolders ? CAMEL_MAPI_STORE_FOLDER_FLAG_FOREIGN_WITH_SUBFOLDERS : 0),
 			foreign_username)) {
 			if (parent_si) {
 				CamelMapiStoreInfo *msi = (CamelMapiStoreInfo *) parent_si;
@@ -151,11 +160,37 @@ add_foreign_folder_to_camel (CamelMapiStore *mapi_store,
 	return res;
 }
 
+static void
+enable_ok_button_by_data (GObject *dialog)
+{
+	GtkEntry *entry;
+	GtkComboBoxText *combo;
+	const gchar *entry_text;
+	gchar *combo_text;
+
+	g_return_if_fail (dialog != NULL);
+
+	entry = g_object_get_data (dialog, STR_USER_NAME_SELECTOR_ENTRY);
+	g_return_if_fail (entry != NULL);
+
+	combo = g_object_get_data (dialog, STR_FOLDER_NAME_COMBO);
+	g_return_if_fail (combo != NULL);
+
+	entry_text = gtk_entry_get_text (entry);
+	combo_text = gtk_combo_box_text_get_active_text (combo);
+
+	gtk_dialog_set_response_sensitive (
+		GTK_DIALOG (dialog), GTK_RESPONSE_OK,
+		entry_text && *entry_text && *entry_text != ' ' && *entry_text != ',' &&
+		combo_text && *combo_text);
+
+	g_free (combo_text);
+}
+
 static void
 name_entry_changed_cb (GObject *dialog)
 {
 	GtkEntry *entry;
-	const gchar *text;
 
 	g_return_if_fail (dialog != NULL);
 
@@ -164,14 +199,20 @@ name_entry_changed_cb (GObject *dialog)
 
 	g_object_set_data (G_OBJECT (entry), STR_MAPI_DIRECT_USER_NAME, NULL);
 
-	text = gtk_entry_get_text (entry);
+	enable_ok_button_by_data (dialog);
+}
 
-	gtk_dialog_set_response_sensitive (GTK_DIALOG (dialog), GTK_RESPONSE_OK, text && *text && *text != ' ' && *text != ',');
+static void
+folder_name_combo_changed_cb (GObject *dialog,
+			      GtkComboBox *combo)
+{
+	enable_ok_button_by_data (dialog);
 }
 
 struct EMapiCheckForeignFolderData
 {
 	GtkWidget *dialog;
+	gboolean include_subfolders;
 	gchar *username;
 	gchar *direct_username;
 	gchar *user_displayname;
@@ -266,6 +307,11 @@ foreign_folder_get_props_cb (EMapiConnection *conn,
 	cffd->folder_container_class = g_strdup (e_mapi_util_find_array_propval (properties, PidTagContainerClass));
 	cffd->parent_folder_id = pid ? *pid : 0;
 
+	if (!cffd->folder_container_class) {
+		/* Default to mail folder */
+		cffd->folder_container_class = g_strdup (IPF_NOTE);
+	}
+
 	return TRUE;
 }
 
@@ -421,6 +467,7 @@ check_foreign_folder_idle (GObject *with_object,
 		cffd->username,
 		cffd->folder_id,
 		cffd->parent_folder_id,
+		cffd->include_subfolders,
 		base_username,
 		base_foldername,
 		perror)) ||
@@ -443,6 +490,24 @@ check_foreign_folder_idle (GObject *with_object,
 	g_free (profile);
 }
 
+static gpointer
+ref_selected_store (GObject *dialog)
+{
+	GtkComboBox *combo_box;
+	CamelStore *store = NULL;
+	GtkTreeIter iter;
+
+	combo_box = g_object_get_data (dialog, STR_ACCOUNTS_COMBO);
+	g_return_val_if_fail (combo_box != NULL, NULL);
+
+	if (gtk_combo_box_get_active_iter (combo_box, &iter)) {
+		gtk_tree_model_get (gtk_combo_box_get_model (combo_box), &iter,
+			COLUMN_STORE, &store, -1);
+	}
+
+	return store;
+}
+
 static void
 subscribe_foreign_response_cb (GObject *dialog,
 			       gint response_id)
@@ -450,6 +515,7 @@ subscribe_foreign_response_cb (GObject *dialog,
 	struct EMapiCheckForeignFolderData *cffd;
 	ENameSelectorEntry *entry;
 	GtkComboBoxText *combo_text;
+	GtkToggleButton *subfolders_check;
 	EDestinationStore *dest_store;
 	CamelStore *cstore;
 	gchar *description;
@@ -465,9 +531,11 @@ subscribe_foreign_response_cb (GObject *dialog,
 
 	entry = g_object_get_data (dialog, STR_USER_NAME_SELECTOR_ENTRY);
 	combo_text = g_object_get_data (dialog, STR_FOLDER_NAME_COMBO);
-	cstore = g_object_get_data (dialog, STR_MAPI_CAMEL_STORE);
+	subfolders_check = g_object_get_data (dialog, STR_SUBFOLDERS_CHECK);
 
 	g_return_if_fail (entry != NULL);
+
+	cstore = ref_selected_store (dialog);
 	g_return_if_fail (cstore != NULL);
 
 	username = NULL;
@@ -517,8 +585,9 @@ subscribe_foreign_response_cb (GObject *dialog,
 	cffd->use_foldername = use_foldername;
 	cffd->folder_id = 0;
 	cffd->parent_folder_id = 0;
+	cffd->include_subfolders = gtk_toggle_button_get_active (subfolders_check);
 
-	description = g_strdup_printf (_("Testing availability of folder “%s” of user “%s”, please wait..."), cffd->orig_foldername, cffd->username);
+	description = g_strdup_printf (_("Testing availability of folder “%s” of user “%s”, please wait…"), cffd->orig_foldername, cffd->username);
 
 	e_mapi_config_utils_run_in_thread_with_feedback (
 		GTK_WINDOW (dialog),
@@ -530,6 +599,7 @@ subscribe_foreign_response_cb (GObject *dialog,
 		e_mapi_check_foreign_folder_data_free);
 
 	g_free (description);
+	g_object_unref (cstore);
 }
 
 static void
@@ -545,15 +615,18 @@ pick_gal_user_clicked_cb (GtkButton *button,
 	g_return_if_fail (dialog != NULL);
 
 	entry = g_object_get_data (dialog, STR_USER_NAME_SELECTOR_ENTRY);
-	mapi_store = g_object_get_data (dialog, STR_MAPI_CAMEL_STORE);
 
 	g_return_if_fail (entry != NULL);
+
+	mapi_store = ref_selected_store (dialog);
 	g_return_if_fail (mapi_store != NULL);
 
 	text = g_strstrip (g_strdup (gtk_entry_get_text (entry)));
 
 	conn = camel_mapi_store_ref_connection (mapi_store, NULL, NULL);
-	if (conn && e_mapi_search_gal_user_modal (GTK_WINDOW (dialog),
+	if (!conn) {
+		e_notice (dialog, GTK_MESSAGE_ERROR, "%s", _("Cannot search for user when the account is offline"));
+	} else if (e_mapi_search_gal_user_modal (GTK_WINDOW (dialog),
 					  conn,
 					  text,
 					  &searched_type,
@@ -568,14 +641,91 @@ pick_gal_user_clicked_cb (GtkButton *button,
 		}
 	}
 
-	if (conn)
-		g_object_unref (conn);
-
+	g_object_unref (mapi_store);
+	g_clear_object (&conn);
 	g_free (text);
 	g_free (display_name);
 	g_free (dn);
 }
 
+static gint
+sort_accounts_by_display_name_cb (gconstpointer ptr1,
+				  gconstpointer ptr2)
+{
+	CamelService *service1 = (CamelService *) ptr1;
+	CamelService *service2 = (CamelService *) ptr2;
+
+	return g_utf8_collate (camel_service_get_display_name (service1), camel_service_get_display_name (service2));
+}
+
+static GtkWidget *
+create_accounts_combo (CamelSession *session,
+		       EClientCache *client_cache,
+		       CamelStore *store)
+{
+	GtkListStore *list_store;
+	GtkTreeIter iter;
+	GtkComboBox *combo_box;
+	ESourceRegistry *registry;
+	GList *services, *link, *accounts = NULL;
+	GtkCellRenderer *renderer;
+
+	list_store = gtk_list_store_new (3,
+		G_TYPE_STRING,		/* COLUMN_UID - UID of the CamelMAPIStore */
+		G_TYPE_STRING,		/* COLUMN_DISPLAY_NAME */
+		CAMEL_TYPE_MAPI_STORE);	/* COLUMN_STORE */
+
+	registry = e_client_cache_ref_registry (client_cache);
+	services = camel_session_list_services (session);
+
+	for (link = services; link; link = g_list_next (link)) {
+		CamelService *service = link->data;
+
+		if (CAMEL_IS_MAPI_STORE (service)) {
+			ESource *source;
+
+			source = e_source_registry_ref_source (registry, camel_service_get_uid (service));
+			if (source && e_source_registry_check_enabled (registry, source)) {
+				accounts = g_list_prepend (accounts, service);
+			}
+
+			g_clear_object (&source);
+		}
+	}
+
+	accounts = g_list_sort (accounts, sort_accounts_by_display_name_cb);
+
+	for (link = accounts; link; link = g_list_next (link)) {
+		CamelService *service = link->data;
+
+		gtk_list_store_append (list_store, &iter);
+		gtk_list_store_set (list_store, &iter,
+			COLUMN_UID, camel_service_get_uid (service),
+			COLUMN_DISPLAY_NAME, camel_service_get_display_name (service),
+			COLUMN_STORE, service,
+			-1);
+	}
+
+	g_list_free_full (services, g_object_unref);
+	g_list_free (accounts);
+	g_clear_object (&registry);
+
+	combo_box = GTK_COMBO_BOX (gtk_combo_box_new_with_model (GTK_TREE_MODEL (list_store)));
+	g_object_unref (list_store);
+
+	renderer = gtk_cell_renderer_text_new ();
+	gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (combo_box), renderer, TRUE);
+	gtk_cell_layout_set_attributes (GTK_CELL_LAYOUT (combo_box), renderer, "text", COLUMN_DISPLAY_NAME, NULL);
+
+	gtk_combo_box_set_id_column (combo_box, COLUMN_UID);
+	if (store)
+		gtk_combo_box_set_active_id (combo_box, camel_service_get_uid (CAMEL_SERVICE (store)));
+	else if (accounts)
+		gtk_combo_box_set_active (combo_box, 0);
+
+	return GTK_WIDGET (combo_box);
+}
+
 /* Opens dialog to subscribe to folders of other
    users in the given store */
 void
@@ -584,23 +734,22 @@ e_mapi_subscribe_foreign_folder (GtkWindow *parent,
 				 CamelStore *store,
                                  EClientCache *client_cache)
 {
-	PangoAttrList *attrs;
 	ENameSelector *name_selector;
 	ENameSelectorModel *name_selector_model;
 	ENameSelectorDialog *name_selector_dialog;
 	GObject *dialog;
 	GtkWidget *content;
-	GtkWidget *label, *widget, *entry;
+	GtkWidget *label, *widget, *entry, *check, *accounts_combo;
 	GtkGrid *grid;
 	GtkComboBoxText *combo_text;
 	gint row;
 
 	g_return_if_fail (session != NULL);
-	g_return_if_fail (store != NULL);
-	g_return_if_fail (CAMEL_IS_MAPI_STORE (store));
+	if (store)
+		g_return_if_fail (CAMEL_IS_MAPI_STORE (store));
 
 	dialog = G_OBJECT (gtk_dialog_new_with_buttons (
-		_("Subscribe to folder of other MAPI user..."),
+		_("Subscribe to folder of other MAPI user…"),
 		parent,
 		GTK_DIALOG_DESTROY_WITH_PARENT,
 		GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE,
@@ -629,18 +778,13 @@ e_mapi_subscribe_foreign_folder (GtkWindow *parent,
 		"halign", GTK_ALIGN_START,
 		NULL);
 
-	attrs = pango_attr_list_new ();
-	pango_attr_list_insert (attrs, pango_attr_weight_new (PANGO_WEIGHT_BOLD));
-	widget = gtk_label_new (camel_service_get_display_name (CAMEL_SERVICE (store)));
+	widget = create_accounts_combo (session, client_cache, store);
 	g_object_set (G_OBJECT (widget),
 		"hexpand", TRUE,
 		"vexpand", FALSE,
-		"use-underline", FALSE,
-		"attributes", attrs,
-		"xalign", 0.0,
 		"halign", GTK_ALIGN_START,
 		NULL);
-	pango_attr_list_unref (attrs);
+	accounts_combo = widget;
 
 	gtk_grid_attach (grid, label, 0, row, 1, 1);
 	gtk_grid_attach (grid, widget, 1, row, 2, 1);
@@ -669,7 +813,7 @@ e_mapi_subscribe_foreign_folder (GtkWindow *parent,
 		"vexpand", FALSE,
 		NULL);
 
-	widget = gtk_button_new_with_mnemonic (_("C_hoose..."));
+	widget = gtk_button_new_with_mnemonic (_("C_hoose…"));
 	g_object_set (G_OBJECT (entry),
 		"hexpand", TRUE,
 		"vexpand", FALSE,
@@ -710,14 +854,22 @@ e_mapi_subscribe_foreign_folder (GtkWindow *parent,
 	gtk_grid_attach (grid, label, 0, row, 1, 1);
 	gtk_grid_attach (grid, widget, 1, row, 2, 1);
 
+	row++;
+
+	check = gtk_check_button_new_with_mnemonic (_("Include _subfolders"));
+	gtk_grid_attach (grid, check, 1, row, 2, 1);
+
 	/* remember widgets for later use */
+	g_object_set_data (dialog, STR_ACCOUNTS_COMBO, accounts_combo);
 	g_object_set_data (dialog, STR_USER_NAME_SELECTOR_ENTRY, entry);
 	g_object_set_data (dialog, STR_FOLDER_NAME_COMBO, widget);
+	g_object_set_data (dialog, STR_SUBFOLDERS_CHECK, check);
 
 	g_object_set_data_full (dialog, STR_MAPI_CAMEL_SESSION, g_object_ref (session), g_object_unref);
-	g_object_set_data_full (dialog, STR_MAPI_CAMEL_STORE, g_object_ref (store), g_object_unref);
 
 	g_signal_connect_swapped (entry, "changed", G_CALLBACK (name_entry_changed_cb), dialog);
+	g_signal_connect_swapped (combo_text, "changed", G_CALLBACK (folder_name_combo_changed_cb), dialog);
+	g_signal_connect_swapped (accounts_combo, "changed", G_CALLBACK (name_entry_changed_cb), dialog);
 
 	name_entry_changed_cb (dialog);
 
